@@ -4,16 +4,21 @@ import pathlib
 import yaml
 import tempfile
 from auterioncli.commands.command_base import CliCommand
-from auterioncli.commands.app_sdk.environment import ensure_docker, ensure_mender_artifact
+from auterioncli.commands.app_sdk.environment import ensure_docker, ensure_mender_artifact, MENDER_CI_TOOLS_TAG
 from auterioncli.commands.app_sdk.slimify import slimify
 import subprocess
 import collections.abc
 import re
 import copy
+import sys
+
+DEVICE_ALIAS = {
+    'ainode': 'jetson'
+}
 
 PLATFORM_ALIAS = {
     'skynode': 'linux/arm64',
-    'ainode': 'linux/arm64'
+    'jetson': 'linux/arm64'
 }
 
 
@@ -33,7 +38,7 @@ def run_command(commands, cwd='.'):
 
 
 def error(msg, code=1):
-    print(msg)
+    print(msg, file=sys.stderr)
     exit(code)
 
 
@@ -49,7 +54,6 @@ class AppBuildCommand(CliCommand):
     def __init__(self, config):
         self._temp_dir = None
         self._config = config
-        self._mender_artifact_path = os.path.join(self._config['persistent_dir'], 'mender-artifact')
 
     def setup_parser(self, parser):
         parser.add_argument('project_dir', help='Location of the project', nargs='?', default='.')
@@ -57,7 +61,7 @@ class AppBuildCommand(CliCommand):
 
     def run(self, args):
         compose_cmd = ensure_docker()
-        ensure_mender_artifact(self._mender_artifact_path)
+        ensure_mender_artifact()
 
         self._temp_dir = tempfile.mkdtemp()
         meta = self._load_metadata(args)
@@ -100,6 +104,20 @@ class AppBuildCommand(CliCommand):
 
         return meta
 
+
+    def _extract_target_devices(self, meta):
+        target_devices = [meta['target-platform']] \
+            if isinstance(meta['target-platform'], str) else meta['target-platform']
+        target_devices = [DEVICE_ALIAS.get(d, d) for d in target_devices]
+        return target_devices
+
+    def _extract_target_platform(self, meta):
+        target_devices = self._extract_target_devices(meta)
+        target_platforms = [PLATFORM_ALIAS.get(d, d) for d in target_devices]
+        if len(set(target_platforms)) > 1:
+            error('Targets have multiple architectures. Aborting.')
+        return target_platforms[0]
+
     def _compose_for_building_from_meta(self, meta):
         api_version = meta['auterion-api-version']
         assert 0 <= api_version <= 2, f'Auterion API version {api_version} is not supported by this ' \
@@ -131,7 +149,7 @@ class AppBuildCommand(CliCommand):
             if 'image' not in service:
                 service['image'] = name + ':' + meta['app-version']
             service['container_name'] = name
-            service['platform'] = PLATFORM_ALIAS.get(meta['target-platform'], meta['target-platform'])
+            service['platform'] = self._extract_target_platform(meta)
         return compose
 
     def _generate_image(self, compose_cmd, args, meta):
@@ -145,16 +163,18 @@ class AppBuildCommand(CliCommand):
 
         # generate a minimal docker-compose file in temp dir for building using docker-compose
         # (don't use self._temp_dir, as podman-compose sets cwd to that directory)
-        compose_file = 'docker-compose-tmp.yml'
+        compose_file = os.path.join(args.project_dir, 'docker-compose-tmp.yml')
         compose = self._compose_for_building_from_meta(meta)
         with open(compose_file, 'w') as f:
             yaml.dump(compose, f)
 
         if not args.skip_docker_build:
             # Just export the required images from the local docker
-            target_platform = PLATFORM_ALIAS.get(meta['target-platform'], meta['target-platform'])
-            run_command(compose_cmd + ['-f', compose_file, 'build'], cwd=args.project_dir)
+            target_platform = self._extract_target_platform(meta)
+            ret = run_command(compose_cmd + ['-f', compose_file, 'build'], cwd=args.project_dir)
 
+            if ret != 0:
+                error("Docker compose build failed.")
             os.remove(compose_file)
 
             non_built_images = [v['image'] for k, v in compose['services'].items() if 'build' not in v]
@@ -220,27 +240,37 @@ class AppBuildCommand(CliCommand):
 
         version = meta['app-version']
         name = meta['app-name']
-        device = meta['target-platform'] if 'target-platform' in meta else 'skynode'
+
+        target_devices = self._extract_target_devices(meta)
+        target_devices_args = []
+        for d in target_devices:
+            target_devices_args += ['-t', d]
+
         out_file = os.path.join(args.project_dir, 'build', name + '.auterionos')
 
         meta_file = os.path.join(args.project_dir, 'auterion-app.yml')
 
         version_file = os.path.join(self._temp_dir, 'version')
-        settings_file = os.path.join(args.project_dir, 'settings.default.env')
-
-        if not os.path.exists(settings_file):
-            settings_file = os.path.join(self._temp_dir, 'settings.default.env')
-            # create empty file
-            pathlib.Path(settings_file).touch()
-
         with open(version_file, 'w') as f:
             f.write(version)
 
+        settings_file = os.path.join(args.project_dir, 'settings.default.env')
+
+        if not os.path.exists(settings_file):
+            settings_file = os.path.abspath(os.path.join(self._temp_dir, 'settings.default.env'))
+            # create empty file
+            pathlib.Path(settings_file).touch()
+
+        # create empty user settings file for AuterionOS < 2.7
+        user_settings_file = os.path.abspath(os.path.join(self._temp_dir, 'settings.user.env'))
+        pathlib.Path(user_settings_file).touch()
+
         file_args = [
-            '-f', meta_file,
-            '-f', image_file,
-            '-f', version_file,
-            '-f', settings_file
+            '-f', os.path.abspath(meta_file),
+            '-f', os.path.abspath(image_file),
+            '-f', os.path.abspath(version_file),
+            '-f', os.path.abspath(settings_file),
+            '-f', os.path.abspath(user_settings_file)
         ]
 
         if meta['auterion-api-version'] < 2:
@@ -250,15 +280,25 @@ class AppBuildCommand(CliCommand):
 
         pathlib.Path(settings_file).touch()
 
-        run_command([
-            self._mender_artifact_path, 'write', 'module-image',
-            '-t', device,
-            '-o', out_file,
+        ret = run_command([
+            'docker',
+            'run',
+            '--rm',
+            '--mount',
+            f'type=bind,source={os.path.abspath(args.project_dir)},target={os.path.abspath(args.project_dir)}',
+            '--mount',
+            f'type=bind,source={os.path.abspath(self._temp_dir)},target={os.path.abspath(self._temp_dir)}',
+            MENDER_CI_TOOLS_TAG,
+            'mender-artifact', 'write', 'module-image',
+            '-o', os.path.abspath(out_file),
             '-T', 'docker',
             '-n', name,
             '--software-filesystem', 'docker-app',
             '--software-name', name,
             '--software-version', version,
-        ] + file_args)
+        ] + target_devices_args + file_args)
+
+        if ret != 0:
+            error('Packaging step failed. Aborting.')
 
 
