@@ -5,6 +5,7 @@ import hashlib
 import sys
 import requests
 import time
+import re
 import pkg_resources
 from packaging import version
 
@@ -70,16 +71,32 @@ class PersistentState:
             f.write(contents)
 
 
-def get_device_serial(address):
+def get_host_device_type():
+    device_type_file = '/persistent/mender/device_type'
+    if os.path.exists(device_type_file):
+        with open(device_type_file, 'r') as f:
+            for l in f.readlines():
+                m = re.match('^device_type=(.*)$', l)
+                return m.group(1)
+    return 'unknown'
+
+
+def get_device_presence(address):
+    """Tries to find device at supplied address. Returns tuple (device_found, serial)"""
     device_info_endpoint = f"http://{address}/api/sysinfo/v1.0/device"
     try:
         response = requests.get(device_info_endpoint, timeout=5)
         if response:
-            return response.json()['uuid']
+            return True, response.json()['uuid']
         else:
-            return None
-    except Exception as e:
-        return None
+            return False, 'Request failed'
+
+    except requests.exceptions.Timeout:
+        # There is something there, but times out
+        return True, ''
+
+    except:
+        return False, 'Request failed'
 
 
 def check_for_updates(persistent_state):
